@@ -1061,11 +1061,39 @@ class HumanNameBruteForceTests(HumanNameTestBase):
 
 class HumanNameConjunctionTestCase(HumanNameTestBase):
     # Last name with conjunction
-    def test117(self):
+    def test_last_name_with_conjunction(self):
         hn = HumanName('Jose Aznar y Lopez')
         self.m(hn.first, "Jose", hn)
         self.m(hn.last, "Aznar y Lopez", hn)
 
+    def test_multiple_conjunctions(self):
+        hn = HumanName("part1 of The part2 of the part3 and part4")
+        self.m(hn.first, "part1 of The part2 of the part3 and part4", hn)
+
+    def test_multiple_conjunctions2(self):
+        hn = HumanName("part1 of and The part2 of the part3 And part4")
+        self.m(hn.first, "part1 of and The part2 of the part3 And part4", hn)
+    
+    def test_ends_with_conjunction(self):
+        hn = HumanName("Jon Dough and")
+        self.m(hn.first, "Jon", hn)
+        self.m(hn.last, "Dough and", hn)
+
+    def test_ends_with_two_conjunctions(self):
+        hn = HumanName("Jon Dough and of")
+        self.m(hn.first, "Jon", hn)
+        self.m(hn.last, "Dough and of", hn)
+
+    def test_starts_with_conjunction(self):
+        hn = HumanName("and Jon Dough")
+        self.m(hn.first, "and Jon", hn)
+        self.m(hn.last, "Dough", hn)
+
+    def test_starts_with_two_conjunctions(self):
+        hn = HumanName("the and Jon Dough")
+        self.m(hn.first, "the and Jon", hn)
+        self.m(hn.last, "Dough", hn)
+
     # Potential conjunction/prefix treated as initial (because uppercase)
     def test_uppercase_middle_initial_conflict_with_conjunction(self):
         hn = HumanName('John E Smith')
@@ -1378,6 +1406,11 @@ class PrefixesTestCase(HumanNameTestBase):
         self.m(hn.first, "Juan", hn)
         self.m(hn.last, "del Sur", hn)
     
+    def test_prefix_with_period(self):
+        hn = HumanName("Jill St. John")
+        self.m(hn.first, "Jill", hn)
+        self.m(hn.last, "St. John", hn)
+    
     def test_prefix_before_two_part_last_name(self):
         hn = HumanName("pennie von bergen wessels")
         self.m(hn.first, "pennie", hn)
